/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.internal.SwapList");

/*
 * Imports 
 */
dojo.require("dojo.i18n");  
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.ListBox");

dojo.requireLocalization("jazz.ui.internal", "SwapListMessages");

/*
 * TODOs 
 */

(function() {

/*
 * Aliases 
 */
var ListBox = jazz.ui.ListBox;

/*
 * Class 
 *
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 */
dojo.declare("jazz.ui.internal.SwapList",
	[dijit._Widget, dijit._Templated], {

	/*
	 * Properties
	 */	
	unselectedListTitleText: "",
	selectedListTitleText: "",

	/*
	 * Specify the entry field to use when sorting list box entries
	 * Options: "label", "value", "id", "type"
	 */		
	sortField: "label",	

	/*  
 	 * Specify icons for selected (left) and unselected (right) bucket 
 	 * entries by providing a map of entry types associated with icon pathes.
 	 * 
 	 * Array iconMap : [Number entryType] = String iconPath
 	 */	
	iconMap: [],

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ui", "internal/templates/SwapList.html"),

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return SwapList
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("jazz.ui.internal", "SwapListMessages");
		this.unselectedListTitleText = this.messages.msgAllItems;
		this.selectedListTitleText = this.messages.msgSelectedItems;
		this.msgAdd = this.messages.msgAdd;
		this.msgAddAll = this.messages.msgAddAll;
		this.msgRemove = this.messages.msgRemove;
		this.msgRemoveAll = this.messages.msgRemoveAll;
		this.msgOutOf = this.messages.msgOutOf;
		
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		// Call super
		this.inherited(arguments);
		
		// Instance Variables //----------------------------------------------->
		this._selectedList = null;
		this._unselectedList = null;

		this.configureUI();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	configureUI: function() {
		// Unselected list title
		this._unselectedListTitle.innerHTML = this.unselectedListTitleText; 
		// Selected list title
		this._selectedListTitle.innerHTML = this.selectedListTitleText;
		
		var self = this;
		
		// Unselected list box
		this._unselectedList = new ListBox({
			height: "280px",
			width: "250px",
			margin: "3px 10px 10px 20px",
			sortField: this.sortField,
			iconMap: this.iconMap,
			ondblClick: function(entry) {
				self._handleEventDoubleClickedUnselected(entry);
			}
		});
		this._unselectedListWidget.appendChild(this._unselectedList.domNode);
		
		// Selected list box		
		this._selectedList = new ListBox({
			height: "280px",
			width: "250px",
			margin: "3px 10px 10px 10px",
			sortField: this.sortField,
			iconMap: this.iconMap,
			ondblClick: function(entry) {
				self._handleEventDoubleClickedSelected(entry);
			}
		});
		this._selectedListWidget.appendChild(this._selectedList.domNode);

		// Add button
		this.connect(this._addButton, "onclick", this._handleEventAddButtonClicked);
		// Add all button
		this.connect(this._addAllButton, "onclick", this._handleEventAddAllButtonClicked);
		// Remove button
		this.connect(this._removeButton, "onclick", this._handleEventRemoveButtonClicked);
		// Remove all button
		this.connect(this._removeAllButton, "onclick", this._handleEventRemoveAllButtonClicked);

	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object entry
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	_handleEventDoubleClickedUnselected: function(entry) {
		// Add entry to selected list
		this._selectedList.addEntry(entry);
		// Remove entry from unselected list
		this._unselectedList.removeEntry(entry);

		this._updateCount();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object entry
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	_handleEventDoubleClickedSelected: function(entry) {
		// Add entry to unselected list
		this._unselectedList.addEntry(entry);
		// Remove entry from selected list
		this._selectedList.removeEntry(entry);

		this._updateCount();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventAddButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get selected entries
		var entries = this._unselectedList.getSelection();
		// Validate entries
		if (entries.length === 0) {
			// NO-OP
			return;
		}
		// Add entries to selected list
		this._selectedList.addEntries(entries);
		// Remove entries from unselected list
		this._unselectedList.removeEntries(entries);

		this._updateCount();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventAddAllButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get all entries
		var entries = this._unselectedList.getAllEntries();
		// Validate entries
		if (entries.length === 0) {
			// NO-OP
			return;
		}
		// Add entries to selected list
		this._selectedList.addEntries(entries);
		// Remove entries from unselected list
		this._unselectedList.removeAllEntries();

		this._updateCount();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventRemoveButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get selected entries
		var entries = this._selectedList.getSelection();
		// Validate entries
		if (entries.length === 0) {
			// NO-OP
			return;
		}
		// Add entries to unselected list
		this._unselectedList.addEntries(entries);
		// Remove entries from selected list
		this._selectedList.removeEntries(entries);

		this._updateCount();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventRemoveAllButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get all entries
		var entries = this._selectedList.getAllEntries();
		// Validate entries
		if (entries.length === 0) {
			// NO-OP
			return;
		}
		// Add entries to unselected list
		this._unselectedList.addEntries(entries);
		// Remove entries from selected list
		this._selectedList.removeAllEntries();

		this._updateCount();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	setSelectedEntries: function(entries) {
		this._selectedList.removeAllEntries();
		this._selectedList.addEntries(entries);
		this._updateCount();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getSelectedEntries: function() {
		return this._selectedList.getAllEntries();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	setUnselectedEntries: function(entries) {
		this._unselectedList.removeAllEntries();
		this._unselectedList.addEntries(entries);
		this._updateCount();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getUnselectedEntries: function() {
		return this._unselectedList.getAllEntries();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param boolean show
 	 * @return void
 	 */	
	showLoading: function(show) {
		this._unselectedList.showLoading(show);
		this._selectedList.showLoading(show);
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_updateCount: function() {
		
		var sCount = this._selectedList.getEntryCount();
		var uCount = this._unselectedList.getEntryCount();
		
		this._selectionCount.innerHTML = sCount;
		this._totalCount.innerHTML = sCount + uCount;
		
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		// Destroy widgets
		this._selectedList.destroy();
		this._unselectedList.destroy();
		
		// Call super
		this.inherited(arguments);
	}
	
});

})();
