/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.OAuthView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.TitlePane");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStep");
dojo.require("com.ibm.team.repository.web.ui.wizard.Wizard");
dojo.require("com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");

var messages;
var setupMessages;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;
var TeamContributorSelectionDialog = com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog;
var STATUS_TYPES = WizardStepHeader.STATUS_TYPES;
var PENDING = "pending";

function clearInlineError(elem) { 
	if(elem) {
		dojo.removeClass(elem,"inlineError");
		elem.innerHTML = "";
	}
};

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.OAuthView", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/OAuthView.html"),

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");
		this.setupMessages = setupMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
	},	
	
	postCreate: function() {
		this.header = new WizardStepHeader({ 
			stepTitle: this.messages.heading,
			reset: dojo.hitch(this,"refresh")
		});
		this.headerArea.appendChild(this.header.domNode);
		
		dojo.connect(this._registerConsumerButton, "onclick", this, "_registerConsumerHandler");
		
		this.connect(this._registerConsumerTable,"onkeyup","handleKeyUp");
		this.requiredInputs = [
			this._consumerNameInput,
			this._consumerSecretInput,
			this._consumerSecretInputAgain
		]
		var self = this;
		dojo.forEach(this.requiredInputs, function(elem) {
			if(elem) {
				self.connect(elem,"onchange","validateChange");
			}
		});
			
		this.refresh();
		
	},
	
	refresh: function() {
		this._getConsumerList();
		this._getProvisionalRegistrations();
		this.header.clearMessages();
		this.clearNewConsumerForm();
	},
	
	clearNewConsumerForm: function () {
		this._consumerSecretInputAgain.value = "";
		this._consumerSecretInput.value = "";
		this._consumerNameInput.value ="";
		this._registerConsumerButton.disabled = true;
		this._consumerTrustedInput.checked = false;
		clearInlineError(this.mismatchErr);
	}, 
	
	_registerConsumerHandler: function () {
		this._registerConsumerButton.disabled = true;

		//Let's just validate one more time before we send out the data.
		var isValid = this.validate();
		if(!isValid) return;
		
		var consumerSecret = this._consumerSecretInput.value;
		var consumerName = this._consumerNameInput.value;
		
		var srh = new ServiceResponseHandler(this, "_handleConsumerRegistrationSuccess", "displayServiceCallbackErrorMessage");
		OAuthAdminClient.registerNewConsumer(srh, consumerName,  consumerSecret);
		return false;
	},
	
	_handleConsumerRegistrationSuccess: function (consumerKey) {
		this.header.showStatus({ 
			type: STATUS_TYPES.OK,
			summary: dojo.string.substitute(this.messages.successMessage, [consumerKey])
		});

		// Do the save in two steps, because it requires 
		// less privledges to save a consumer than it 
		// does to make a consumer trusted
		if (this._consumerTrustedInput.checked) {
			var srh = new ServiceResponseHandler(this, "_handleConsumerTrustSuccess", "displayServiceCallbackErrorMessage");
			OAuthAdminClient.setConsumerTrusted(srh, consumerKey, true);		
		} else {
			this.clearNewConsumerForm();
			this._getConsumerList();
		}
		
	},
	
	_handleConsumerTrustSuccess: function () {
		this.clearNewConsumerForm();
		this._getConsumerList();
	},
	
	_getConsumerList: function () {
		dojox.data.dom.removeChildren(this.consumersTableBody);	
		dojo.toggleClass(this.noConsumer, "noDisplay", true);
		var srh = new ServiceResponseHandler(this, "displayConsumersList", "displayServiceCallbackErrorMessage");
		OAuthAdminClient.getConsumers(srh);
	},
	
	_deleteConsumer: function(e) {
		var verifyDelete = confirm(this.messages.confirmDelete);
        if (verifyDelete) {
			var consumerKey = e.target.id;
			consumerKey = consumerKey.substring(4);
			this._keyPendingDeletion = consumerKey;
			var srh = new ServiceResponseHandler(this, "removeConsumerEntry", "displayServiceCallbackErrorMessage");
			OAuthAdminClient.deleteConsumer(srh, consumerKey);
		}
	},
	
	removeConsumerEntry: function() {
		var elem = dojo.byId("row-" + this._keyPendingDeletion);
		this.consumersTableBody.removeChild(elem);
		
	},
	
	sortConsumers: function () {	
		return this.consumerList.sort(function(a, b){
			var aName = a.name ? a.name.toLowerCase() : "";
			var bName = b.name ? b.name.toLowerCase() : "";
			return aName > bName ? 1 : -1;
		});
	},
	
	displayConsumersList: function (consumers) {
		if (consumers){
			this.consumerList = consumers;
			if(this.consumerList.length > 1) {
				this.sortConsumers();
			}
			var self = this;
			dojo.forEach(this.consumerList, function(consumer){
                self.addConsumerListRow(consumer);
            });
		}
		dojo.toggleClass(this.consumerTable, "noDisplay", !consumers);
		dojo.toggleClass(this.noConsumer, "noDisplay", consumers);
	},
	
	addConsumerListRow: function(consumer) {
		var row = document.createElement("tr");
		row.setAttribute("id","row-" + consumer.key);
		
		var nameCell = document.createElement("td");
		var name = this._getEditConsumerLink(consumer);
		nameCell.appendChild(name);
		row.appendChild(nameCell);
		this._consumerNameInput.value = "";
		
		var cell2 = document.createElement("td");
		cell2.innerHTML = consumer.key;
		row.appendChild(cell2);
		
		var cell2B = document.createElement("td");
		var userId = document.createTextNode(consumer.userId ? consumer.userId : "");
		cell2B.appendChild(userId);
		row.appendChild(cell2B);
		
		var cell3 = document.createElement("td");
		cell3.innerHTML = consumer.trusted ? this.messages.trustedValue : this.messages.untrustedValue;
		row.appendChild(cell3);
			
		var cell4 = document.createElement("td");
		
		//Build delete button
        var del = document.createElement("img");
        var removeIconSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove.gif");
        del.setAttribute('src', removeIconSrc);
		del.setAttribute('id', "del-" + consumer.key);
		del.setAttribute('title', this.messages.removeConsumerButton);
        
		cell4.appendChild(del);
		row.appendChild(cell4);
		
		this.consumersTableBody.appendChild(row);
		dojo.connect(del,"onclick",this, "_deleteConsumer");
		dojo.addClass(del,"delButton");
	},
	
	_getEditConsumerLink: function (consumer) {
		var aElement = document.createElement("a");
		aElement.onclick = dojo.hitch(this, function() {
			this._launchEditConsumer(consumer);
		});
		aElement.className ="nameLink";
		aElement.innerHTML = consumer.name;
		return aElement;
	},
	
	_launchEditConsumer: function (consumer) {
		var caller = this;
		var step1 = new com.ibm.team.repository.web.ui.internal.admin.server.EditConsumerStep({
			stepTitle: this.messages.editConsumerDialogTitle,
			consumer: consumer, 
			caller: caller
		});

		var wizardArgs = {
			steps: [step1],
			primaryTitle: dojo.string.substitute(this.messages.editConsumerHeading, [consumer.key]),
			showStepSummary: false,
			processFinish: function(){
				this._steps[0].saveUpdates();
			},
			processCancel: function() {		
				if (this._steps[0].hasUpdates()) {
					var verifyCancel = confirm(caller.messages.confirmEditCancel)
					if (verifyCancel) {
						this.container.viewBox.close();
					}
				} else {
					this.container.viewBox.close();
				}				
			}, 	
			saveSuccess: function () {
				caller._getConsumerList();
				caller.header.clearMessages();
				this.container.viewBox.close();
			}
		}
		this.wizard = new com.ibm.team.repository.web.ui.wizard.Wizard(wizardArgs);
					
	},
	
	_getProvisionalRegistrations: function () {
		// clear table & no provisional msg
		dojox.data.dom.removeChildren(this.provisionalTableBody);
		dojo.toggleClass(this.noProvisional, "noDisplay", true);
		
		var srh = new ServiceResponseHandler(this, "displayProvisionalRegistrations", "displayServiceCallbackErrorMessage");
		OAuthAdminClient.getProvisionalRegistrations(srh);
	},
	
	displayProvisionalRegistrations: function (provisional) {
		if (provisional){
			this.provisionalKeys = provisional;
			var self = this;
			dojo.forEach(this.provisionalKeys, function(key){
                self.addProvisionalRegistrationRow(key);
            });
		}
		dojo.toggleClass(this.provisionalTable, "noDisplay", !provisional);
		dojo.toggleClass(this.noProvisional, "noDisplay", provisional);
		dojo.toggleClass(this.provisionalSaveButton, "noDisplay", !provisional);
	},
	
	addProvisionalRegistrationRow: function(provisionalKey) {
		var row = document.createElement("tr");

		var nameCell = document.createElement("td");
		var name = document.createElement("input");
		name.value = provisionalKey.name;
		name.type = "text";
		name.setAttribute("id","name-" + provisionalKey.key);
		nameCell.appendChild(name);
		row.appendChild(nameCell);
		provisionalKey._nameCell = nameCell;
		
		var cell1 = document.createElement("td");
		var key = document.createTextNode(provisionalKey.key);
		cell1.appendChild(key);
		row.appendChild(cell1);
		
		var cell1B = document.createElement("td");
		var userId = document.createTextNode(provisionalKey.userId ? provisionalKey.userId : "");
		cell1B.appendChild(userId);
		row.appendChild(cell1B);
		
		var cell2 = document.createElement("td");
		var trusted = document.createTextNode(provisionalKey.trusted ? messages.trustedValue : messages.untrustedValue);
		cell2.appendChild(trusted);
		row.appendChild(cell2);
		
		var cell3 = document.createElement("td");
		var grantTrusted = document.createElement("input");
		grantTrusted.type = "checkbox";
		provisionalKey._grantTrusted = grantTrusted;
		cell3.appendChild(grantTrusted);
		row.appendChild(cell3);
		grantTrusted.checked = provisionalKey.trusted;
		provisionalKey._cell3 = cell3;
		
		var cell4 = document.createElement("td");
		var select = this.createAcceptSelect(provisionalKey);
		provisionalKey._acceptSelect = select;
		dojo.connect(provisionalKey._acceptSelect, "onchange", this, "handleProvisionalChange");
		cell4.appendChild(select);
		row.appendChild(cell4);
		provisionalKey._cell4 = cell4;

		this.provisionalTableBody.appendChild(row);
	},
	
	createAcceptSelect: function (provisionalKey) {
		var node = document.createElement("select");
		var option = document.createElement("option");
		option.text = this.messages.pendingLabel;
		option.value = PENDING;
		node.options[node.options.length] = option;
		option = document.createElement("option");
		option.text = this.messages.approveLabel;
		option.value = "true";
		node.options[node.options.length] = option;
		option = document.createElement("option");
		option.text = this.messages.rejectLabel;
		option.value = "false";
		node.options[node.options.length] = option;
		return node;
	},
	
	handleProvisionalChange: function () {
		var dirty = false;
		for (var i = 0; i < this.provisionalKeys.length; i++) {
			if (this.provisionalKeys[i]._acceptSelect && this.provisionalKeys[i]._acceptSelect.value != PENDING) {
				dirty = true;
				break;
			}
		}
		this.provisionalSaveButton.disabled = !dirty;
	},		
			
	saveProvisional: function () {
		var self = this;
		this.provisionalSaveButton.disabled = true;
		var openRequests = 0;
		dojo.forEach(this.provisionalKeys, function(provisionalKey){   
			if (provisionalKey._acceptSelect && provisionalKey._acceptSelect.value != PENDING) {
				provisionalKey.grant = provisionalKey._grantTrusted.checked
				var acceptProvisional = {
					key: provisionalKey,
					
					displayAcceptedRegistration: function(response){
						openRequests--;
					
						var trust = this.key.grant ? self.messages.trustedValue : self.messages.untrustedValue;
						this.key._cell3.replaceChild(document.createTextNode(trust), this.key._cell3.firstChild);
						this.key._grantTrusted = null;
						
						var approval = provisionalKey._acceptSelect.value === "true" ? self.messages.approvedLabel : self.messages.rejectedLabel;
						this.key._cell4.replaceChild(document.createTextNode(approval), this.key._cell4.firstChild);
						this.key._acceptSelect = null;	
						
						var name = provisionalKey.name;
						this.key._nameCell.replaceChild(document.createTextNode(name), this.key._nameCell.firstChild);
						if(openRequests == 0) {
							self._getConsumerList();
						}
						
					},
					handleAcceptError: function(err){
						self.displayServiceCallbackErrorMessage(err);
					}
				}
				provisionalKey.name = dojo.byId("name-" + provisionalKey.key).value;
				openRequests++;
				var srh = new ServiceResponseHandler(acceptProvisional, "displayAcceptedRegistration", "handleAcceptError");
				OAuthAdminClient.acceptProvisionalRegistration(srh, provisionalKey.name, provisionalKey.key, provisionalKey._grantTrusted.checked, provisionalKey._acceptSelect.value);
			}		
		});
		
	},
	
	displayServiceCallbackErrorMessage: function(errorObj) {
		this.header.displayServiceCallbackErrorMessage(errorObj);
	},
	
	validateChange: function () {
		this.setRegisterButton(this.validate());
	},
	
	validate: function() {
		// change this function with care.  It is also used directly by the EditConsumerStep below, so make sure "this" works for both contexts	
		
		//Let's first assume we are valid ("ready").
		var isReady = true;
		clearTimeout(this.onKeyUpTimout);
		
		//Make sure none of the required input fields is blank
		dojo.forEach(this.requiredInputs, function(elem) {
			if(elem && elem.value ==  "") {
				isReady = false;
			} 
		});
					
		//Do the secrets match?
		if(isReady && this._consumerSecretInput.value != this._consumerSecretInputAgain.value) {
			isReady = false;
			dojo.addClass(this.mismatchErr,"inlineError");
			this.mismatchErr.innerHTML = this.setupMessages.secretMismatch;
			this.header.showStatus({ 
				type: STATUS_TYPES.ERROR,
				summary: this.setupMessages.correctFields
			});
		} else {
			clearInlineError(this.mismatchErr);
		}
		
		if(isReady) {			
			this.header.clearMessages();
		}
		return isReady;
	},
	
	setRegisterButton: function (valid) {
		if(!valid) {			
			this._registerConsumerButton.disabled = true;	
		} else {
			this._registerConsumerButton.disabled = false;
		}
	},
	
	handleKeyUp: function(e) {
		if(e.target && e.target.tagName === "INPUT") {
			if(this.onKeyUpTimout){
				clearTimeout(this.onKeyUpTimout);
			}
			var f = dojo.hitch(this, this.validateChange);
			var timeout = 1500;
			if (e.target._jazzRow  && e.target._jazzRow.hasError) {
				timeout = 500;
			}
			this.onKeyUpTimout = setTimeout(f, timeout);
		}
	}

});

var WizardStep = com.ibm.team.repository.web.ui.wizard.WizardStep;
dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.EditConsumerStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/EditOAuthConsumer.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");
		this.setupMessages = setupMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
	},	

	postCreate: function () {
		this.header =  new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		this.validate= this.caller.validate;
		this.handleKeyUp = this.caller.handleKeyUp;

		
		this.connect(this._selectUserLink,"onclick", function (e) {
			dojo.stopEvent(e);
			var userPicker = new TeamContributorSelectionDialog({ allowMultiple : false });
			this.connect(userPicker,"onAdd","associateUser");
		});
		
		this.connect(this._clearConsumerName,"onclick", function (e) {
			dojo.stopEvent(e);
			this.clearSelectedUser();
		});
		
		this.connect(this._editConsumerTable,"onkeyup","handleKeyUp");
		this.connect(this._consumerSecretInput,"onchange","secretValueChanged");
		this.connect(this._consumerTrustedInput,"onclick","validateChange");

		this.requiredInputs = [
			this._consumerNameInput,
			this._consumerSecretInput,
			this._consumerSecretInputAgain
		]	
		var self = this;
		dojo.forEach(this.requiredInputs, function(elem) {
			if(elem) {
				self.connect(elem,"onchange","validateChange");
			}
		});

		this.secretUpdated = false;
		this._consumerTrustedInput.checked = this.consumer.trusted;
		if (this.consumer.userId) {
			this.associateUser([{name: this.consumer.userId, userId:this.consumer.userId }]);
		}

	}, 
	
	validateChange: function () {
		this.triggerButtonStateUpdate();
	},
	
	secretValueChanged: function () {
		this.secretUpdated = true;
	}, 
		
	associateUser: function (newUsers) {
		if (newUsers != null && newUsers[0] != null) {
			this.associatedUser = newUsers[0];
			this._selectedConsumerName.innerHTML = newUsers[0].name;
			dojo.toggleClass(this._selectUserLink, "noDisplay", true);
			dojo.toggleClass(this._clearConsumerName, "noDisplay", false);
			this.validateChange();
		}
	}, 
	
	clearSelectedUser: function () {
		this.associatedUser = null;
		this._selectedConsumerName.innerHTML = "";
		dojo.toggleClass(this._selectUserLink, "noDisplay", false);
		dojo.toggleClass(this._clearConsumerName, "noDisplay", true);
		this.validateChange();
	}, 
	
	saveUpdates: function () {
		var isValid = this.validate();
		if(!isValid) return;
		
		var args = {
			consumerKey: this.consumer.key,
			trusted: this._consumerTrustedInput.checked
		};
		if (this.consumer.name != this._consumerNameInput.value) {
			args.name = this._consumerNameInput.value;
		}
		if (this.secretUpdated) {
			args.consumerSecret = this._consumerSecretInput.value;
		}
		var newUser = this.associatedUser ? this.associatedUser.userId : null;
		if (this.consumer.userId != newUser) {
			args.userId = newUser ? newUser : "";
		}
			
		var srh = new ServiceResponseHandler(this, "handleConsumerUpdateSuccess", "displayServiceCallbackErrorMessage");
		OAuthAdminClient.updateConsumer(srh, args);
	},
		
	handleConsumerUpdateSuccess: function () {
		this.wizard.saveSuccess();		
	},
	
	displayServiceCallbackErrorMessage: function(errorObj) {
		this.header.displayServiceCallbackErrorMessage(errorObj);
	},
	
	canFinish: function(){
		var isValid = this.validate();
		return isValid && this.hasUpdates();
	}, 
	
	hasUpdates: function () {
		if (this.consumer.name != this._consumerNameInput.value) {
			return true;
		}
		if (this.secretUpdated) {
			return true;
		}
		var newUser = this.associatedUser ? this.associatedUser.userId : null;
		if (this.consumer.userId != newUser) {
			return true;
		}
		if (this.consumer.trusted != this._consumerTrustedInput.checked) {
			return true;
		}
		return false;
	}
	

});

})();