/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.ConfigureDatabase");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigureDatabaseMessages");

(function() {

var ServerPropertyServiceRow = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow;
var WizardConfigurationStep = com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep;
var licensing = com.ibm.team.repository.web.client.internal.licensing;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;

var DB_SERVICE_NAME = "com.ibm.team.repository.service.internal.rdb.ConnectionPoolService"
var DB_VENDOR_NAME = "db.vendor";
var DB_TYPE_NAME = "db.type";
var JDBC_TYPE_VALUE = "JDBC";
var VENDOR_LICENSE_CHECK_PREFIX = "com.ibm.team.repository.support.";
	
var DISPLAY_PROPS = [];
DISPLAY_PROPS.J2EE = [
	"db.j2ee.dataSource"
];
DISPLAY_PROPS.JDBC = [
	"db.jdbc.password",
	"db.jdbc.location"
];

var SUPPORTED_VENDORS = [
	{propertyValue: "DB2", displayValue: "DB2", licenseCheck: "db2"}, 
	{propertyValue: "DERBY", displayValue: "Derby", licenseCheck: "derby"}, 
	{propertyValue: "ORACLE", displayValue: "Oracle", licenseCheck: "oracle"}, 
	{propertyValue: "SQLSERVER", displayValue: "SQL Server", licenseCheck: "sqlserver"}	
];

//Example location strings
var LOCATION_STRINGS = [
	{vendorName: "DB2", locString: "//localhost:50000/JAZZ:user=db2inst1;password={password};"},  
	{vendorName: "ORACLE", locString: "thin:jazzDBuser/{password}@//localhost:1521/ORCL"}, 
	{vendorName: "SQLSERVER", locString: "//localhost:1433;databaseName=jazz;user=jazzDBuser;password={password}"},
	{vendorName: "DB2i", locString: "//hostname;hold statements=true;user=jazzDBuser;password={password};"}, 
	{vendorName: "DB2z", locString: "//ipAddress:ipPort/location:user=jazzDBuser;password={password};"}
];


var messages;

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.ConfigureDatabase",  WizardConfigurationStep, {

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigureDatabaseMessages");
		this.implementationClassName = "com.ibm.team.repository.service.internal.rdb.ConnectionPoolService"
		this.pendingNext = null;
		this.stepTitle = messages.wizardPageHeading;
		this.description = messages.wizardPageDescription;	
	},
	
	createInnerStepUI: function () {
		this.requestLicenseRestrictions();
		this.dbConfigUI = new DBConfigInnerSteps();
		this.serviceContainer = this.dbConfigUI._hiddenTableBody;
		this.configArea.appendChild(this.dbConfigUI.domNode);
		dojo.connect(this.dbConfigUI.createDBLink,"onclick",this,"_createDatabases");
	},

	requestLicenseRestrictions: function () {
		this.licenseRequestPending = true;
		this.licenseCheckResponse = 0;
		this.licenseCheckRequests = 0;
		var self = this;
		dojo.forEach(SUPPORTED_VENDORS, function(vendor){ 
	  		self.licenseCheckRequests++;
			var checkingLicense = {
				vendor: vendor,
				
				handleLicenseCheck: function(response){
					vendor.licenseAllows = response.code;
					self.incrementLicneseCheckCount();
				},
				handleAcceptError: function(err){
					// TODO make more generic error path in config step
					self.saveError(err);
				}
			}
			var srh = new ServiceResponseHandler(checkingLicense, "handleLicenseCheck", "handleAcceptError");
			licensing.checkLicense(srh, {operationId: VENDOR_LICENSE_CHECK_PREFIX + vendor.licenseCheck} );
		});
	},
	
	incrementLicneseCheckCount: function () {
		this.licenseCheckResponse++;
		if (this.licenseCheckResponse === this.licenseCheckRequests){
			// All license checks are back. Load the vendors if the UI already loaded 
			this.licenseRequestPending = false;
			if (this.displayloaded && !this.vendorsDisplayed) {
				this._displayVendorsAllowedByLicense();
			}
		}	
	},
		
	_initializePropertiesAfterLoaded: function () {
		this._initializeHiddenProperty(DB_VENDOR_NAME);
		this._initializeHiddenProperty(DB_TYPE_NAME);
		
		this._initializePropertyDisplay(this.dbConfigUI._jdbcEditPropertiesBody, DISPLAY_PROPS.JDBC);
		this._initializePropertyDisplay(this.dbConfigUI._j2eeEditPropertiesBody, DISPLAY_PROPS.J2EE);
				
		this._displayRelevantSections();
		
		dojo.connect(this.dbConfigUI._selectVendor, "onchange", this, "_handleVendorSelectChange");
		dojo.connect(this.dbConfigUI._selectType, "onchange", this, "_handleTypeSelectChange");
		
		this.inherited(arguments);
		if (!this.licenseRequestPending) {
			this._displayVendorsAllowedByLicense();
		}
		this.displayloaded = true;
	},	
	
	_displayVendorsAllowedByLicense: function () {
		this.vendorsDisplayed = true;
		var self = this;
		dojo.forEach(SUPPORTED_VENDORS, function(vendor){ 
	  		if (vendor.licenseAllows && vendor.licenseAllows === "OK"){
				var newOption = new Option(vendor.displayValue, vendor.propertyValue);
				self.dbConfigUI._selectVendor.options[self.dbConfigUI._selectVendor.options.length] = newOption;
			} else {
				self._displayLicenseLimitedMessage();
			}
		});	
		this._synchronizeSelectValue();	
	},

	_displayLicenseLimitedMessage: function() {
		dojo.toggleClass(this.dbConfigUI._licenseLimitedMessage, "noDisplay", false);
	}, 
	
	_initializeHiddenProperty: function (propertyName) {
		var targetProperty = this._config.getServerPropertyElement(propertyName);
		dojo.connect(targetProperty, "_markInvalid", this, function (msg) {
			this._handleInvalidInput(msg);
			this._showHiddenError(msg);
		});
	},
	
	_showHiddenError: function(msg){
		if (msg.code === "ERROR") {
			this._setState(this.STATUS.PENDING_ERR, [msg]);
		} else {
			this._setState(this.STATUS.PENDING_WARN, [msg]);
		}
	},
	
	_initializePropertyDisplay: function (attachPoint, displayProps) {
		
		var targetPropertyServiceRow = new ServerPropertyServiceRow({
			_enableToggle: false,
			rowClass: ["indentNone editProperties"],
			_descriptionMode: "table"
		});
		attachPoint.appendChild(targetPropertyServiceRow.domNode);
		
		var longExists = false;
		var shortExists = false;
		for (var i=0; i < displayProps.length; i++) {
			var propertyElement = this._config.getServerPropertyElement(displayProps[i]);
			if (propertyElement.elementInfo.propertyType === "STRING") {
				longExists = true;
				var table = targetPropertyServiceRow._longPropertiesTableBody;
			} else {
				shortExists = true;
				var table = targetPropertyServiceRow._shortPropertiesTableBody;
			}
			table.appendChild(propertyElement.domNode);
		};
		if (!longExists) {
			targetPropertyServiceRow._longPropertiesTableHeader.style.display = "none";
		}
		if (!shortExists) {
			targetPropertyServiceRow._shortPropertiesTableHeader.style.display = "none";
		}
	},
		
	_handleVendorSelectChange: function (event) {
		this._handleSelectChange(event, DB_VENDOR_NAME);
	},
	
	_handleTypeSelectChange: function(event){
		this._handleSelectChange(event, DB_TYPE_NAME);
	}, 
	
	_handleSelectChange: function (event, propertyName) {
		var newSelectValue = event.target.value;
		this._handleChange(newSelectValue, propertyName);
	},
	
	_handleChange: function (newValue, propertyName) {
		var targetProperty = this._config.getServerPropertyElement(propertyName);
		
		targetProperty._editValueNode.value = newValue;
		
		if (targetProperty._savedValue != newValue) {
			targetProperty._markChanged();
		} else {
			targetProperty._markUnchanged();
		}
		targetProperty._checkValidity();
		this._displayRelevantSections();
	},
	
	_revert: function() {
		this.inherited(arguments);
		this._synchronizeSelectValue();
		this._displayRelevantSections();

	},
			
	// UPDATE VIEW METHODS	
	_displayRelevantSections: function() {
		// KMB TODO need to clear out values on hide
		if (this._config.getEditedValueForProperty(DB_TYPE_NAME) === JDBC_TYPE_VALUE){
			this.dbConfigUI._j2eePropertiesSection.style.display = "none";
			this.dbConfigUI._jdbcPropertiesSection.style.display = "block";
			//KMB TODO show vendor props if jdbc
		} else {
			this.dbConfigUI._j2eePropertiesSection.style.display = "block";
			this.dbConfigUI._jdbcPropertiesSection.style.display = "none";
		}
		
		this._updateLocationStringExample();
		
		this.dbConfigUI._createTablesSection.style.display = "none";	
		
	}, 
	
	_allowPropertyEdit: function (allowEdits){
		this.inherited(arguments); 
		this.dbConfigUI._selectVendor.disabled = !allowEdits;
		this.dbConfigUI._selectType.disabled = !allowEdits;
	},
	
	_synchronizeSelectValue: function() {
		var vendorProp = this._config.getServerPropertyElement(DB_VENDOR_NAME);
// 20090416 do we still have any licenses that don't allow Derby?!  Think I can remove this code
//		if (vendorProp._editValueNode.value === "DERBY" && !licenseAllowsVendor("DERBY")) {
//			this._handleChange("DB2", DB_VENDOR_NAME);
//		}
		this.dbConfigUI._selectVendor.value = vendorProp._editValueNode.value;
		var typeProp = this._config.getServerPropertyElement(DB_TYPE_NAME);
		this.dbConfigUI._selectType.value = typeProp._editValueNode.value;
	},
	
	_testConnectionCallback: function(msg) {
		var connected = null;
		this.dbConfigUI._createTablesSection.style.display = "none";
		if (msg != null) {
			connected = this.STATUS.CONNECTED_WARN;
			for (var i = 0; i < msg.length; i++) {
				if (msg[i].code === "ERROR") {
					connected = this.STATUS.CONNECTED_ERR;
					//Check to see if we need to create Tables (i.e. error code 3)
					if(msg[i].id == 3) {
						msg[i].message = this.messages.DBneedsTables;
						this.dbConfigUI.createDBLink.disabled = false;
						this.dbConfigUI._createTablesSection.style.display = "block";
						this.dbConfigUI.progressBox.style.display = "none";	
					}
				}
			}
		} else {
			connected = this.STATUS.CONNECTED_OK;
		}
		if (this._initialLoadedState == null) {
			this._initialLoadedState = connected;
		}
		this._setState(connected, msg);
	},
	
	_createDatabases: function() {
		this.dbConfigUI.createDBLink.disabled = true;

		this.STATUS.DB_SUCCESS = {
				test: false,
				revert: false,
				next: true,
				edit: true,
				code: "OK",
				resolvedMessage: messages.creationSuccess
		};
		this.STATUS.DB_FAIL = {
				test: true,
				revert: true,
				next: false,
				edit: true,
				code: "ERROR",
				resolvedMessage: messages.creationFail
		};
		this.STATUS.DB_PROGRESS = {
				test: false,
				revert: false,
				next: false,
				edit: false,
				code: "INFO",
				resolvedMessage: messages.creationProgress
		};
		
		//First we need to save the properties before we can call createTables
		this.invokeSave(this, "saveOK", "saveError");
	},
	
	saveOK: function() {
		this.header.showStatus({type: "OK",	summary: this.messages.settingsSaved});
		var srh = new ServiceResponseHandler(this, "_handleCreateTables", "_handleAcceptError");
		ServerStatusClient.postInitalizeDatabase(srh);
	},
	
	_handleCreateTables: function(response){	
		if(response == "true") { //Response passed back as string
		
			//Database tables are not created.
			this._setState(this.STATUS.DB_PROGRESS);
			this.dbConfigUI.progressBox.style.display = "inline";
			//Begin to poll for progress
			this.pollRequests = 0;
			this.pollID = setInterval(dojo.hitch(this,"checkProgress"),500);
		} else {
			this.header.showStatus({type: "INFO",	summary: this.messages.tablesExist});
		}
	},
	

	
	_handleAcceptError: function(err){
		this.saveError(err);
	},
	
	checkProgress: function() {
		
		//Don't want to keep sending out requests unless they are coming back...
		if(this.pollRequests < 1) {
			var self = this;
			var pollWorker = {
				handleSuccess: function(response){
					self.pollRequests--;
					if(response.id == 6) {
						//The database creation has completed sucessfully
						self.dbConfigUI.progressBox.style.display = "none";
						clearInterval(self.pollID);
						self.dbConfigUI.createDBLink.disabled = true;
						self._setState(self.STATUS.DB_SUCCESS);
						
					} else if(response.id == 2) {
						//The database creation failed with an exception
						self.dbConfigUI.progressBox.style.display = "none";
						clearInterval(self.pollID);
						self.dbConfigUI.createDBLink.disabled = false;
						
						//Convert to the expected errorObj format
						var errorObj = {
							message: response.message,
							exception: response.stacktrace
						};
						
						self._setState(self.STATUS.DB_FAIL,errorObj,true);
					} 
				},
				handleError: function(err){
					self.pollRequests--;
					clearInterval(self.pollID);
					self.saveError(err);
				}
			}
			this.pollRequests++;
			var srh = new ServiceResponseHandler(pollWorker, "handleSuccess", "handleError");
			ServerStatusClient.getCreationStatus(srh);
		}

	},
	
	_updateLocationStringExample: function() {
		var locString = "";
		var selectedVendor = this._config.getEditedValueForProperty(DB_VENDOR_NAME);
		if(selectedVendor != "DERBY") {
			dojo.some(LOCATION_STRINGS,function(vendor) {
				if(selectedVendor == vendor.vendorName) {
					locString = vendor.locString;
					return;
				}
			});
			var messageString = dojo.string.substitute(this.messages.jdbcLocExample, [ locString ]);
			this._config.setServerPropertyExample("db.jdbc.location", messageString);
		} else {
			this._config.setServerPropertyExample("db.jdbc.location","");
		}
	}
	
});	

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.DBConfigInnerSteps", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/ConfigureDatabase.html"),
	
	constructor: function(){
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigureDatabaseMessages");
		var JAZZ_ROOT = net.jazz.ajax._contextRoot;
		var adminLink = "<a href='" + JAZZ_ROOT + "/admin#action=com.ibm.team.repository.admin.manageLicenses'>";
		this.reviewAdminLink = dojo.string.substitute(this.messages.reviewAdminLink, [ adminLink, "</a>" ]);
	}
	
});

var DBConfigInnerSteps = com.ibm.team.repository.web.ui.internal.wizard.DBConfigInnerSteps;



})();